#
# passwordDialog.py - GUI front end code for setting the root password
#
# Brent Fox <bfox@redhat.com>
# Damien Durand <splinux@fedoraproject.org>
# Haikel Guemar <karlthered@fedoraproject.org>
#
# Copyright 2002 - 2006 Red Hat, Inc. and Fedora Usability
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import string
import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
from gi.repository import Gtk,Gdk
import libuser
import pwquality
import sys
sys.path.append('/usr/share/system-config-rootpassword')

##
## I18N
## 
import gettext
t = gettext.translation("system-config-rootpassword", fallback=True)
_ = t.gettext

##
## Icon for windows
##

iconPixbuf = None      
img = Gtk.Image()
img.set_from_file("/usr/share/system-config-rootpassword/pixmaps/system-config-rootpassword.png")
iconPixbuf = img.get_pixbuf()

class childWindow:
    #You must specify a runPriority for the order in which you wish your module to run
    runPriority = 40
    moduleName = str(_("Root Password"))
    moduleClass = "reconfig"
    commentTag = _("Change the root password for the system")

    def destroy(self, args):
        Gtk.main_quit()
    
    def __init__(self, user=None):
        self.admin = libuser.ADMIN()
        self.user = user
        self.doDebug = None
        self.mainWindow = Gtk.Window()
        self.mainWindow.connect("destroy", self.destroy)
        self.mainWindow.set_position(Gtk.WindowPosition.CENTER)
        self.mainWindow.set_icon(iconPixbuf)
        self.mainWindow.set_border_width(10)
        self.mainWindow.set_resizable(False)
        self.toplevel = Gtk.VBox(False)
        self.mainVBox = Gtk.VBox(False)
        self.iconBox = Gtk.HBox()
        self.passwordEntry = Gtk.Entry()
        self.passwordEntry.set_visibility(False)
        self.confirmEntry = Gtk.Entry()
        self.confirmEntry.set_visibility(False)
        self.okButton = Gtk.Button(_('OK'))
        self.cancelButton = Gtk.Button(_('Cancel'))
        self.securityBar = Gtk.ProgressBar()
        self.securityBar.set_show_text(True)

        p = None
        self.icon = None
        img = Gtk.Image()
        img.set_from_file("../pixmaps/system-config-rootpassword.png")
        p = img.get_pixbuf()
        if p == None:
            img.set_from_file("/usr/share/system-config-rootpassword/pixmaps/system-config-rootpassword.png")
            p=img.get_pixbuf()
        if p == None:
            print("no image found")
        else:
            self.icon = Gtk.Image()
            self.icon.set_from_pixbuf(p)

        self.title = Gtk.Label(_("Root Password"))
        color = Gdk.RGBA()
        color.parse("white")
        self.title.override_color(Gtk.StateType.NORMAL,color)

        if self.user == None:
            #If no user was passed in, assume that we're chaning the root password
            self.msgLabel = Gtk.Label(_("You can change the root password by filling \nthe following fields:"))
            self.user = "root"
        else:
            self.msgLabel = Gtk.Label(_("Please set the password for %s.") % self.user)
        self.msgLabel.set_line_wrap(False)
        self.msgLabel.set_alignment(0.0, 0.5)

        self.table = Gtk.Table(2, 2)
        label = Gtk.Label(_("New Root Password:"))
        label.set_alignment(0.0, 0.5)
        self.table.attach(label, 0, 1, 0, 1)
        self.table.attach(self.passwordEntry, 1, 2, 0, 1)
        label = Gtk.Label(_("Confirm Root Password:"))
        label.set_alignment(0.0, 0.5)
        self.table.attach(label, 0, 1, 1, 2)
        self.table.attach(self.confirmEntry, 1, 2, 1, 2)

    def okClicked(self, *args):
        result = self.apply()
        if result == 0:
            Gtk.main_quit()

    def apply(self, *args):
        passwd1 = self.passwordEntry.get_text()
        passwd2 = self.confirmEntry.get_text()

        if self.doDebug:
            print("don't change root password in debug mode")
            return 0

        #if (len(passwd1) < 6) or (len(passwd2) < 6):
	#	self.msgLabel.set_text("For security reasons, the root password\n should have at least 6 characters in length.")
	#	self.passwordEntry.set_text("")
	#	self.confirmEntry.set_text("")
	#	self.grabFocus()
        #   	return None

        elif passwd1 != passwd2:
                self.msgLabel.set_text("The two fields did not match. Please try \nagain:")
                self.passwordEntry.set_text("")
                self.confirmEntry.set_text("")
                self.grabFocus()
                return None


        if self.doDebug:
            return 0
        
        userEnt = self.admin.lookupUserByName(self.user)
        self.admin.setpassUser(userEnt, passwd1, 0)
        return 0

    def setKickstartData(self, kickstartData):
        print("in setKickstartData")
        #Get the lang from the list of languages
        return 0

    def setKickstartDefaults(self, kickstartData):
        pass


    def launch(self, doDebug = None):
        self.doDebug = doDebug

        self.internalVBox = Gtk.VBox(False, 10)
        self.internalVBox.set_border_width(10)

        self.msgLabel.set_size_request(500, -1)
        self.internalVBox.pack_start(self.msgLabel, False)
        self.internalVBox.pack_start(self.table, False)
        self.mainVBox.pack_start(self.internalVBox, True)
        self.passwordEntry.grab_focus()
        return self.mainVBox, self.icon, self.moduleName

    def grabFocus(self):
        self.passwordEntry.grab_focus()
        
    def activation(self, *args):     
        length_password = len(self.passwordEntry.get_text())
        length_confirm = len(self.confirmEntry.get_text())  
        if (length_password == 0):
        	test = 0.0
        else :
                test = ((length_password/16.0)<= 1.0) and (length_password/16.0) or 1.0
        strength = None
        try:
            settings = pwquality.PWQSettings()
            settings.read_config()
            settings.check(self.passwordEntry.get_text())
        except pwquality.PWQError as e:
            strength = e.args[1]
        if (strength == None):
            self.securityBar.set_text("Security Level: Strong")
            if (test < 0.8):
            	test = 0.8
            self.securityBar.set_fraction(test)
            self.passwordEntry.set_tooltip_text("Good")
        else :
            self.passwordEntry.set_tooltip_text(str(strength))
            if (0.5 >= test ):
                self.securityBar.set_text("Security Level: Weak")
                self.securityBar.set_fraction(test)
               
            if (0.8 >= test > 0.5):
                self.securityBar.set_text("Security Level: Medium")
                self.securityBar.set_fraction(test)
                if (test > 0.8):
                    self.securityBar.set_fraction(0.8)          
        if (length_password < 6) | (length_confirm < 6):
            self.okButton.set_sensitive(False)
        if length_confirm == length_password:
            self.okButton.set_sensitive(True)
        else:
            self.okButton.set_sensitive(False)


    def stand_alone(self):
        self.mainWindow.set_title(_("Root Password"))
        bb = Gtk.HButtonBox()
        bb.set_layout(Gtk.ButtonBoxStyle.END)
        bb.set_spacing(5)    
        self.passwordEntry.connect("changed", self.activation)
        self.confirmEntry.connect("changed", self.activation)
        self.okButton.set_sensitive(False)
        self.okButton.connect("clicked", self.okClicked) 
        self.cancelButton.connect("clicked", self.destroy)  
        self.passwordEntry.set_tooltip_text("Remember that your password should have at least 6 characters in length")            
        self.securityBar.set_fraction(0.0) 
        if self.icon:
            self.iconBox.pack_start(self.icon, False,0,0)
        self.iconBox.pack_start(self.msgLabel, False,0,0)
        self.mainWindow.add(self.toplevel)
        self.mainVBox.set_spacing(5)
        self.toplevel.pack_start(self.iconBox, False,0,0)
        self.toplevel.pack_start(self.mainVBox, False,0,0)
        self.mainVBox.pack_start(self.table, False,0,0)
        self.mainVBox.pack_start(self.securityBar,True,0,0)
        bb.pack_start(self.cancelButton,True,0,0)
        bb.pack_start(self.okButton,True,0,0)
        self.mainVBox.pack_start(bb, False,0,0)
        self.mainWindow.show_all()
        self.passwordEntry.grab_focus()
        Gtk.main()
